"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var IConfiguration_1 = require("./IConfiguration");
var Configuration_1 = require("./Configuration");
var core_1 = require("@ts-ioc/core");
var platform_server_1 = require("@ts-ioc/platform-server");
var index_1 = require("./core/index");
var index_2 = require("./aop/index");
var Log4jsAdapter_1 = require("./logAdapter/Log4jsAdapter");
var aop_1 = require("@ts-ioc/aop");
var logs_1 = require("@ts-ioc/logs");
/**
 * mvc applaction bootstrap.
 *
 * @export
 * @class Bootstrap
 */
var Bootstrap = /** @class */ (function (_super) {
    tslib_1.__extends(Bootstrap, _super);
    /**
     * Creates an instance of WebApplication.
     * @param {string} rootdir
     * @param {Type<any>} [appType]
     * @memberof WebApplication
     */
    function Bootstrap(rootdir) {
        var _this = _super.call(this, rootdir) || this;
        _this.middlewares = [];
        _this.beforeSMdls = [];
        _this.afterSMdls = [];
        return _this;
    }
    /**
     * create new application.
     *
     * @static
     * @param {string} rootdir
     * @returns
     * @memberof WebApplication
     */
    Bootstrap.create = function (rootdir) {
        return new Bootstrap(rootdir);
    };
    /**
     * use middleware `fn` or  `MiddlewareFactory`.
     *
     * @param {(IMiddleware | Middleware | Token<any>)} middleware
     * @returns {this}
     * @memberof Bootstrap
     */
    Bootstrap.prototype.use = function (middleware) {
        this.middlewares.push(middleware);
        return this;
    };
    Bootstrap.prototype.useServer = function (middleware, afterMvc) {
        if (afterMvc === void 0) { afterMvc = true; }
        if (afterMvc) {
            this.afterSMdls.push(middleware);
        }
        else {
            this.beforeSMdls.push(middleware);
        }
        return this;
    };
    Bootstrap.prototype.useListener = function (listener) {
        this._listener = listener;
    };
    /**
     * use boostrap to start application.
     *
     * @template T
     * @param {Type<T>} [appModule]
     * @returns {Promise<T>}
     * @memberof Bootstrap
     */
    Bootstrap.prototype.run = function (appModule) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                return [2 /*return*/, this.bootstrap(appModule)];
            });
        });
    };
    /**
     * bootstrap mvc application with App Module.
     *
     * @template T
     * @param {Token<T>} [appModule]
     * @returns {Promise<T>}
     * @memberof Bootstrap
     */
    Bootstrap.prototype.bootstrap = function (appModule) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var appType, app, cfg, container, config, server, port;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        appType = appModule || index_1.Application;
                        return [4 /*yield*/, _super.prototype.bootstrap.call(this, appType)];
                    case 1:
                        app = _a.sent();
                        if (!core_1.isFunction(app.getHttpServer) || !core_1.isFunction(app.use) || !core_1.isFunction(app.getServer) || !core_1.isFunction(app.setup)) {
                            throw new Error('configuration bootstrap or bootstrap with module is not right application implements IApplication.');
                        }
                        return [4 /*yield*/, this.getConfiguration()];
                    case 2:
                        cfg = _a.sent();
                        return [4 /*yield*/, this.getContainer()];
                    case 3:
                        container = _a.sent();
                        container.bindProvider(index_1.ApplicationToken, app);
                        app.setup(this.beforeSMdls, this.afterSMdls);
                        config = container.get(IConfiguration_1.ConfigurationToken);
                        server = app.getHttpServer();
                        port = config.port || parseInt(process.env.PORT || '0');
                        if (config.hostname) {
                            server.listen(port, config.hostname, this._listener);
                        }
                        else {
                            server.listen(port, this._listener);
                        }
                        console.log('service listen on port: ', port);
                        return [2 /*return*/, app];
                }
            });
        });
    };
    Bootstrap.prototype.setConfigRoot = function (config) {
        config.rootdir = config.rootdir ? platform_server_1.toAbsolutePath(this.rootdir, config.rootdir) : this.rootdir;
    };
    Bootstrap.prototype.getDefaultConfig = function () {
        return new Configuration_1.Configuration();
    };
    Bootstrap.prototype.getModuleConfigure = function (builder, appModule) {
        var cfg = builder.getConfigure(appModule, index_1.AppModule);
        if (core_1.lang.hasField(cfg)) {
            return cfg;
        }
        return _super.prototype.getModuleConfigure.call(this, builder, appModule);
    };
    Bootstrap.prototype.initContainer = function (config, container) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var modules, controllers, aops, servers, _a, _b;
            return tslib_1.__generator(this, function (_c) {
                switch (_c.label) {
                    case 0:
                        if (!container.has(aop_1.AopModule)) {
                            container.register(aop_1.AopModule);
                        }
                        if (!container.has(logs_1.LogModule)) {
                            container.register(logs_1.LogModule);
                        }
                        return [4 /*yield*/, _super.prototype.initContainer.call(this, config, container)];
                    case 1:
                        _c.sent();
                        container.bindProvider(IConfiguration_1.ConfigurationToken, config);
                        container.bindProvider(Configuration_1.Configuration, config);
                        this.registerDefaults(container);
                        if (!this.middlewares) return [3 /*break*/, 3];
                        return [4 /*yield*/, (_a = this.builder).loadModule.apply(_a, [container].concat(this.middlewares.filter(function (m) { return core_1.isClass(m); })))];
                    case 2:
                        _c.sent();
                        _c.label = 3;
                    case 3:
                        if (!config.middlewares) return [3 /*break*/, 5];
                        return [4 /*yield*/, this.builder.loadModule(container, {
                                basePath: config.rootdir,
                                files: config.middlewares
                            })];
                    case 4:
                        modules = _c.sent();
                        if (!config.useMiddlewares || config.useMiddlewares.length < 1) {
                            config.useMiddlewares = this.middlewares.concat(modules);
                        }
                        else {
                            config.useMiddlewares = this.middlewares.concat(config.useMiddlewares);
                        }
                        _c.label = 5;
                    case 5:
                        if (!config.controllers) return [3 /*break*/, 7];
                        return [4 /*yield*/, this.builder.loadModule(container, {
                                basePath: config.rootdir,
                                files: config.controllers
                            })];
                    case 6:
                        controllers = _c.sent();
                        if (!config.useControllers || config.useControllers.length < 1) {
                            config.useControllers = controllers;
                        }
                        _c.label = 7;
                    case 7:
                        if (config.logConfig) {
                            container.registerSingleton(logs_1.LogConfigureToken, config.logConfig);
                        }
                        if (config.debug) {
                            container.register(index_2.DebugLogAspect);
                        }
                        container.register(index_2.AuthAspect);
                        if (!config.aop) return [3 /*break*/, 9];
                        return [4 /*yield*/, this.builder.loadModule(container, {
                                basePath: config.rootdir,
                                files: config.aop
                            })];
                    case 8:
                        aops = _c.sent();
                        config.usedAops = aops;
                        _c.label = 9;
                    case 9: return [4 /*yield*/, (_b = this.builder).loadModule.apply(_b, [container].concat(this.beforeSMdls.concat(this.afterSMdls).filter(function (m) { return core_1.isClass(m); })))];
                    case 10:
                        servers = _c.sent();
                        if (servers && servers.length) {
                            config.usedServerMiddlewares = servers;
                        }
                        return [2 /*return*/, container];
                }
            });
        });
    };
    Bootstrap.prototype.registerDefaults = function (container) {
        index_1.registerDefaults(container);
        container.register(Log4jsAdapter_1.Log4jsAdapter);
    };
    Bootstrap.classAnnations = { "name": "Bootstrap", "params": { "constructor": ["rootdir"], "create": ["rootdir"], "use": ["middleware"], "useServer": ["middleware", "afterMvc"], "useListener": ["listener"], "run": ["appModule"], "bootstrap": ["appModule"], "setConfigRoot": ["config"], "getDefaultConfig": [], "getModuleConfigure": ["builder", "appModule"], "initContainer": ["config", "container"], "registerDefaults": ["container"] } };
    return Bootstrap;
}(platform_server_1.ServerApplicationBuilder));
exports.Bootstrap = Bootstrap;

//# sourceMappingURL=sourcemaps/Bootstrap.js.map
