/// <reference types="koa" />
/// <reference types="node" />
import * as Koa from 'koa';
import { IContainer, InjectToken, Token } from '@ts-ioc/core';
import * as http from 'http';
import * as https from 'https';
import { IConfiguration } from '../IConfiguration';
import { ILogger, ILoggerManager } from '@ts-ioc/logs';
import { IServerMiddleware, ServerMiddleware } from './servers/index';
import { IMiddleware } from './middlewares/index';
import { IApplication } from './IApplication';
/**
 * Application of type mvc.
 *
 * @export
 * @class Application
 * @extends {Koa}
 */
export declare class Application implements IApplication {
    private httpServer;
    private koa;
    private _loggerMgr;
    container: IContainer;
    configuration: IConfiguration;
    constructor();
    getServer(): Koa;
    getLoggerManger(): ILoggerManager;
    getLogger(name?: string): ILogger;
    getHttpServer(): http.Server | https.Server;
    use(middleware: Koa.Middleware): void;
    geDefaultMiddlewares(): InjectToken<IMiddleware>[];
    setup(beforeSMdls: (ServerMiddleware | Token<IServerMiddleware>)[], afterSMdls: (ServerMiddleware | Token<IServerMiddleware>)[]): void;
    setupRoutes(config: IConfiguration): void;
    setupMiddlewares(middlewares: Token<any>[], filter?: (token: Token<any>) => boolean): void;
    protected registerDefaultMiddlewars(): void;
    protected getExcludeMiddwares(cfg: IConfiguration): Token<any>[];
    protected setupServerMiddwares(middlewares: (ServerMiddleware | Token<IServerMiddleware>)[]): void;
    static classAnnations: any;
}
