"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var aop_1 = require("@ts-ioc/aop");
var Koa = require("koa");
var core_1 = require("@ts-ioc/core");
var http = require("http");
// import * as http2 from 'http2';
var https = require("https");
var IConfiguration_1 = require("../IConfiguration");
var logs_1 = require("@ts-ioc/logs");
var index_1 = require("./middlewares/index");
var index_2 = require("./router/index");
var index_3 = require("./middlewares/index");
/**
 * Application of type mvc.
 *
 * @export
 * @class Application
 * @extends {Koa}
 */
var Application = /** @class */ (function () {
    function Application() {
    }
    Application.prototype.getServer = function () {
        if (!this.koa) {
            this.koa = new Koa();
        }
        return this.koa;
    };
    Application.prototype.getLoggerManger = function () {
        if (!this._loggerMgr) {
            var cfg = this.configuration;
            this._loggerMgr = this.container.resolve(logs_1.ConfigureLoggerManagerToken, { config: cfg.logConfig });
        }
        return this._loggerMgr;
    };
    Application.prototype.getLogger = function (name) {
        return this.getLoggerManger().getLogger(name);
    };
    Application.prototype.getHttpServer = function () {
        if (!this.httpServer) {
            var cfg = this.configuration;
            if (cfg.httpsOptions) {
                this.httpServer = https.createServer(cfg.httpsOptions, this.getServer().callback);
            }
            else {
                this.httpServer = http.createServer(this.getServer().callback());
            }
        }
        return this.httpServer;
    };
    Application.prototype.use = function (middleware) {
        this.getServer().use(middleware);
    };
    Application.prototype.geDefaultMiddlewares = function () {
        this.registerDefaultMiddlewars();
        return [
            index_1.BodyParserMiddlewareToken,
            index_1.JsonMiddlewareToken,
            index_1.LogMiddlewareToken,
            index_1.SessionMiddlewareToken,
            index_1.ContentMiddlewareToken,
            index_1.ContextMiddlewareToken,
            index_1.CorsMiddlewareToken,
            index_1.ViewsMiddlewareToken
        ];
    };
    Application.prototype.setup = function (beforeSMdls, afterSMdls) {
        this.setupServerMiddwares(beforeSMdls);
        var cfg = this.configuration;
        var excludes = this.getExcludeMiddwares(cfg);
        var filter = function (excludes) {
            return function (m) {
                if (excludes.length) {
                    return excludes.indexOf(m) < 0;
                }
                return true;
            };
        };
        this.setupMiddlewares(cfg.beforeMiddlewares, filter(excludes));
        this.setupMiddlewares(this.geDefaultMiddlewares(), filter(excludes));
        this.setupMiddlewares(cfg.useMiddlewares, filter(excludes.concat(cfg.afterMiddlewares)));
        this.setupRoutes(cfg);
        this.setupMiddlewares(cfg.afterMiddlewares, filter(excludes));
        this.setupServerMiddwares(afterSMdls);
    };
    Application.prototype.setupRoutes = function (config) {
        var router = this.container.get(config.routerMiddlewate || index_2.RouterMiddlewareToken);
        router.register.apply(router, config.useControllers);
        router.setup();
    };
    Application.prototype.setupMiddlewares = function (middlewares, filter) {
        var _this = this;
        if (!middlewares || middlewares.length < 1) {
            return;
        }
        middlewares.forEach(function (m) {
            if (!m) {
                return;
            }
            if (filter && !filter(m)) {
                return;
            }
            if (core_1.isToken(m)) {
                var middleware = _this.container.get(m);
                if (core_1.isFunction(middleware.setup)) {
                    middleware.setup();
                }
            }
            else if (core_1.isFunction(m)) {
                _this.use(m);
            }
        });
    };
    Application.prototype.registerDefaultMiddlewars = function () {
        this.container.register(index_3.DefaultContentMiddleware);
        this.container.register(index_3.DefaultContextMiddleware);
        this.container.register(index_3.DefaultJsonMiddleware);
        this.container.register(index_3.DefaultLogMiddleware);
        this.container.register(index_3.DefaultSessionMiddleware);
        this.container.register(index_3.DefaultBodyParserMiddleware);
        this.container.register(index_3.DefaultViewsMiddleware);
        this.container.register(index_3.DefaultCorsMiddleware);
        this.container.register(index_2.Router);
    };
    Application.prototype.getExcludeMiddwares = function (cfg) {
        return cfg.excludeMiddlewares || [];
    };
    Application.prototype.setupServerMiddwares = function (middlewares) {
        var _this = this;
        if (!middlewares || middlewares.length < 1) {
            return;
        }
        middlewares.forEach(function (m) {
            if (!m) {
                return;
            }
            if (core_1.isToken(m)) {
                var middleware = _this.container.get(m);
                if (core_1.isFunction(middleware.setup)) {
                    middleware.setup();
                }
            }
            else if (core_1.isFunction(m)) {
                m(_this, _this.container);
            }
        });
    };
    Application.classAnnations = { "name": "Application", "params": { "constructor": [], "getServer": [], "getLoggerManger": [], "getLogger": ["name"], "getHttpServer": [], "use": ["middleware"], "geDefaultMiddlewares": [], "setup": ["beforeSMdls", "afterSMdls"], "setupRoutes": ["config"], "setupMiddlewares": ["middlewares", "filter"], "registerDefaultMiddlewars": [], "getExcludeMiddwares": ["cfg"], "setupServerMiddwares": ["middlewares"] } };
    tslib_1.__decorate([
        core_1.Inject(core_1.ContainerToken),
        tslib_1.__metadata("design:type", Object)
    ], Application.prototype, "container", void 0);
    tslib_1.__decorate([
        core_1.Inject(IConfiguration_1.ConfigurationToken),
        tslib_1.__metadata("design:type", Object)
    ], Application.prototype, "configuration", void 0);
    Application = tslib_1.__decorate([
        aop_1.NonePointcut,
        core_1.Singleton,
        tslib_1.__metadata("design:paramtypes", [])
    ], Application);
    return Application;
}());
exports.Application = Application;

//# sourceMappingURL=../sourcemaps/core/Application.js.map
