"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var core_1 = require("@ts-ioc/core");
var index_1 = require("./results/index");
var IContext_1 = require("./IContext");
/**
 * Base Controller.
 *
 * @export
 * @class Controller
 */
var BaseController = /** @class */ (function () {
    function BaseController() {
    }
    /**
     * respone view result.
     *
     * @param {string} viewName
     * @param {object} [model]
     * @returns
     * @memberof BaseController
     */
    BaseController.prototype.view = function (viewName, model) {
        return new index_1.ViewResult(viewName, model);
    };
    /**
     * respone file result.
     *
     * @param {(string | Buffer | Stream)} file
     * @param {string} [contentType]
     * @param {string} [fileDownloadName]
     * @returns
     * @memberof BaseController
     */
    BaseController.prototype.file = function (file, contentType, fileDownloadName) {
        return new index_1.FileResult(file, contentType, fileDownloadName);
    };
    /**
     * respone redirect result.
     * Perform a 302 redirect to `url`.
     *
     * The string "back" is special-cased
     * to provide Referrer support, when Referrer
     * is not present `alt` or "/" is used.
     *
     * Examples:
     *
     *    this.redirect('back');
     *    this.redirect('back', '/index.html');
     *    this.redirect('/login');
     *    this.redirect('http://google.com');
     * @param {string} url
     * @param {string} [alt]
     * @returns
     * @memberof BaseController
     */
    BaseController.prototype.redirect = function (url, alt) {
        return new index_1.RedirectResult(url, alt);
    };
    /**
     * respone json result.
     *
     * @param {object} data
     * @returns
     * @memberof BaseController
     */
    BaseController.prototype.json = function (data) {
        return new index_1.JsonResult(data);
    };
    BaseController.classAnnations = { "name": "BaseController", "params": { "constructor": [], "view": ["viewName", "model"], "file": ["file", "contentType", "fileDownloadName"], "redirect": ["url", "alt"], "json": ["data"] } };
    tslib_1.__decorate([
        core_1.Inject(IContext_1.ContextToken),
        tslib_1.__metadata("design:type", Object)
    ], BaseController.prototype, "context", void 0);
    BaseController = tslib_1.__decorate([
        core_1.Injectable,
        tslib_1.__metadata("design:paramtypes", [])
    ], BaseController);
    return BaseController;
}());
exports.BaseController = BaseController;

//# sourceMappingURL=../sourcemaps/core/BaseController.js.map
