"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var ResultValue_1 = require("./ResultValue");
var stream_1 = require("stream");
var IConfiguration_1 = require("../../IConfiguration");
var core_1 = require("@ts-ioc/core");
var fs_1 = require("fs");
var path_1 = require("path");
var index_1 = require("../../errors/index");
/**
 * controller method return result type of file.
 *
 * @export
 * @class FileResult
 */
var FileResult = /** @class */ (function (_super) {
    tslib_1.__extends(FileResult, _super);
    function FileResult(file, contentType, fileDownloadName) {
        var _this = _super.call(this, contentType) || this;
        _this.file = file;
        _this.fileDownloadName = fileDownloadName;
        return _this;
    }
    FileResult.prototype.sendValue = function (ctx, container) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var _this = this;
            var defer, file, contentType, confige, filepath;
            return tslib_1.__generator(this, function (_a) {
                defer = core_1.Defer.create();
                file = this.file;
                contentType = this.contentType;
                confige = container.get(IConfiguration_1.ConfigurationToken);
                if (core_1.isString(file)) {
                    filepath = path_1.join(confige.rootdir, file);
                    if (fs_1.existsSync(filepath)) {
                        fs_1.readFile(filepath, contentType || 'utf8', function (err, data) {
                            if (err) {
                                defer.reject(err);
                            }
                            else {
                                defer.resolve(new Buffer(data));
                            }
                        });
                    }
                }
                else if (file instanceof Buffer) {
                    defer.resolve(file);
                }
                else if (file instanceof stream_1.Stream) {
                    file.once('end', function (data) {
                        defer.resolve(new Buffer(data));
                    })
                        .once('error', defer.reject);
                }
                return [2 /*return*/, defer.promise
                        .then(function (buffer) {
                        if (_this.fileDownloadName) {
                            ctx.attachment(_this.fileDownloadName);
                        }
                        ctx.body = buffer;
                    }, function (err) {
                        throw new index_1.BadRequestError(err.toString());
                    })];
            });
        });
    };
    FileResult.classAnnations = { "name": "FileResult", "params": { "constructor": ["file", "contentType", "fileDownloadName"], "sendValue": ["ctx", "container"] } };
    return FileResult;
}(ResultValue_1.ResultValue));
exports.FileResult = FileResult;

//# sourceMappingURL=../../sourcemaps/core/results/FileResult.js.map
