"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var BaseRoute_1 = require("./BaseRoute");
var core_1 = require("@ts-ioc/core");
var index_1 = require("../decorators/index");
var index_2 = require("../decorators/index");
var IAuthorization_1 = require("../IAuthorization");
var index_3 = require("../../errors/index");
var util_1 = require("util");
var index_4 = require("../results/index");
var RequestMethod_1 = require("../RequestMethod");
var IConfiguration_1 = require("../../IConfiguration");
var ModelParser_1 = require("./ModelParser");
var ControllerRoute = /** @class */ (function (_super) {
    tslib_1.__extends(ControllerRoute, _super);
    function ControllerRoute(route, controller) {
        var _this = _super.call(this, route) || this;
        _this.controller = controller;
        return _this;
    }
    ControllerRoute.prototype.options = function (container, ctx, next) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var err_1;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        _a.trys.push([0, 2, , 3]);
                        return [4 /*yield*/, this.invokeOption(ctx, container, next)];
                    case 1:
                        _a.sent();
                        return [3 /*break*/, 3];
                    case 2:
                        err_1 = _a.sent();
                        if (err_1 instanceof index_3.HttpError) {
                            ctx.status = err_1.code;
                            ctx.message = err_1.message;
                        }
                        else {
                            ctx.status = 500;
                            console.error(err_1);
                        }
                        return [3 /*break*/, 3];
                    case 3: return [2 /*return*/];
                }
            });
        });
    };
    ControllerRoute.prototype.navigating = function (container, ctx, next) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var err_2;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        _a.trys.push([0, 4, , 5]);
                        if (!(ctx.method !== 'OPTIONS')) return [3 /*break*/, 2];
                        return [4 /*yield*/, this.invoke(ctx, container)];
                    case 1:
                        _a.sent();
                        return [3 /*break*/, 3];
                    case 2: throw new index_3.ForbiddenError();
                    case 3: return [2 /*return*/, next()];
                    case 4:
                        err_2 = _a.sent();
                        if (err_2 instanceof index_3.HttpError) {
                            ctx.status = err_2.code;
                            ctx.message = err_2.message;
                        }
                        else {
                            ctx.status = 500;
                            console.error(err_2);
                        }
                        return [3 /*break*/, 5];
                    case 5: return [2 /*return*/];
                }
            });
        });
    };
    ControllerRoute.prototype.invokeOption = function (ctx, container, next) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var requestOrigin, origin, headersSet, set, config, options, coremeta, maxAge, allowsM, allowH;
            return tslib_1.__generator(this, function (_a) {
                requestOrigin = ctx.get('Origin');
                ctx.vary('Origin');
                if (!requestOrigin) {
                    return [2 /*return*/, next()];
                }
                origin = requestOrigin;
                headersSet = {};
                set = function (key, value) {
                    ctx.set(key, value);
                    headersSet[key] = value;
                };
                config = container.get(IConfiguration_1.ConfigurationToken);
                options = config.corsOptions || {};
                if (ctx.method !== 'OPTIONS') {
                    set('Access-Control-Allow-Origin', origin);
                    if (options.credentials === true) {
                        set('Access-Control-Allow-Credentials', 'true');
                    }
                    if (options.exposeHeaders) {
                        set('Access-Control-Expose-Headers', options.exposeHeaders);
                    }
                    if (!options.keepHeadersOnError) {
                        return [2 /*return*/, next()];
                    }
                    return [2 /*return*/, next().catch(function (err) {
                            err.headers = Object.assign({}, err.headers, headersSet);
                            throw err;
                        })];
                }
                else {
                    coremeta = this.getCorsMeta(ctx, container, ctx.get('Access-Control-Request-Method'));
                    // console.log('coremeta', coremeta);
                    if (!coremeta) {
                        return [2 /*return*/, next()];
                    }
                    options = Object.assign({}, options, coremeta);
                    ctx.set('Access-Control-Allow-Origin', origin);
                    if (options.credentials === true) {
                        ctx.set('Access-Control-Allow-Credentials', 'true');
                    }
                    maxAge = String(options.maxAge);
                    if (maxAge) {
                        ctx.set('Access-Control-Max-Age', maxAge);
                    }
                    allowsM = core_1.isArray(options.allowMethods) ?
                        options.allowMethods.map(function (m) { return RequestMethod_1.methodToString(m); }).filter(function (m) { return !!m; }).join(',')
                        : options.allowMethods;
                    allowsM = allowsM || 'GET,HEAD,PUT,POST,DELETE,PATCH';
                    if (allowsM) {
                        ctx.set('Access-Control-Allow-Methods', allowsM);
                    }
                    allowH = core_1.isArray(options.allowHeaders) ? options.allowHeaders.filter(function (m) { return !!m; }).join(',') : options.allowHeaders;
                    if (!allowH) {
                        allowH = ctx.get('Access-Control-Request-Headers');
                    }
                    if (allowH) {
                        ctx.set('Access-Control-Allow-Headers', allowH);
                    }
                    ctx.status = 204;
                }
                return [2 /*return*/];
            });
        });
    };
    ControllerRoute.prototype.getCorsMeta = function (ctx, container, reqMethod) {
        if (!reqMethod) {
            return null;
        }
        var methodCors = core_1.getMethodMetadata(index_1.Cors, this.controller);
        var method = RequestMethod_1.parseRequestMethod(reqMethod);
        var meta = this.getRouteMetaData(ctx, container, method);
        if (meta && meta.propertyKey) {
            var corsmetas = core_1.getMethodMetadata(index_1.Cors, this.controller)[meta.propertyKey] || [];
            if (corsmetas.length < 1) {
                corsmetas = core_1.getTypeMetadata(index_1.Cors, this.controller);
            }
            if (corsmetas.length) {
                return corsmetas.find(function (cor) {
                    // console.log('find CorsMetadata:', cor);
                    if (!cor.allowMethods) {
                        return true;
                    }
                    if (cor.allowMethods) {
                        if (core_1.isString(cor.allowMethods)) {
                            return cor.allowMethods.toUpperCase().indexOf(reqMethod) >= 0;
                        }
                        else if (core_1.isArray(cor.allowMethods)) {
                            return cor.allowMethods.some(function (c) { return RequestMethod_1.methodToString(c) === reqMethod; });
                        }
                    }
                    return false;
                });
            }
        }
        return null;
    };
    ControllerRoute.prototype.invoke = function (ctx, container) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var meta, ctrl, hasAuth, auth, lifeScope, params, providers, response, contentType;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        meta = this.getRouteMetaData(ctx, container, RequestMethod_1.parseRequestMethod(ctx.method));
                        if (!(meta && meta.propertyKey)) return [3 /*break*/, 8];
                        ctrl = container.get(this.controller);
                        if (container.has(IAuthorization_1.AuthorizationToken)) {
                            hasAuth = core_1.hasClassMetadata(index_2.Authorization, ctrl) || core_1.hasMethodMetadata(index_2.Authorization, ctrl, meta.propertyKey);
                            if (hasAuth) {
                                auth = container.get(IAuthorization_1.AuthorizationToken);
                                if (!auth.isAuth()) {
                                    throw new index_3.UnauthorizedError();
                                }
                            }
                        }
                        lifeScope = container.getLifeScope();
                        params = lifeScope.getMethodParameters(this.controller, ctrl, meta.propertyKey);
                        providers = this.createProvider(container, ctx, ctrl, meta, params);
                        return [4 /*yield*/, container.invoke.apply(container, [this.controller, meta.propertyKey, ctrl].concat(providers))];
                    case 1:
                        response = _a.sent();
                        if (!core_1.isPromise(response)) return [3 /*break*/, 3];
                        return [4 /*yield*/, response];
                    case 2:
                        response = _a.sent();
                        _a.label = 3;
                    case 3:
                        contentType = meta.contentType;
                        if (!(core_1.isString(response) || core_1.isBoolean(response) || core_1.isNumber(response) || core_1.isArray(response) || core_1.isDate(response) || util_1.isBuffer(response))) return [3 /*break*/, 4];
                        ctx.body = util_1.isBuffer(response) ? Buffer.from(response) : response;
                        return [3 /*break*/, 7];
                    case 4:
                        if (!core_1.isObject(response)) return [3 /*break*/, 7];
                        if (!(response instanceof index_4.ResultValue)) return [3 /*break*/, 6];
                        return [4 /*yield*/, response.sendValue(ctx, container)];
                    case 5:
                        _a.sent();
                        return [3 /*break*/, 7];
                    case 6:
                        ctx.body = response;
                        _a.label = 7;
                    case 7: return [3 /*break*/, 9];
                    case 8: throw new index_3.NotFoundError();
                    case 9: return [2 /*return*/];
                }
            });
        });
    };
    ControllerRoute.prototype.createProvider = function (container, ctx, ctrl, meta, params) {
        var parser = container.get(ModelParser_1.ModelParser);
        if (params && params.length) {
            var restParams_1 = {};
            if (this.isRestUri(meta.route)) {
                var routes_1 = meta.route.split('/').map(function (r) { return r.trim(); });
                var restParamNames = routes_1.filter(function (d) { return /^\S*:/.test(d); }); // .map(rest => rest.substring(1));
                var baseURL = this.cutEmptyPath(this.url, true);
                var routeUrls_1 = this.cutEmptyPath(ctx.url.replace(baseURL, '')).split('/');
                restParamNames.forEach(function (pname) {
                    var val = routeUrls_1[routes_1.indexOf(pname)];
                    restParams_1[pname.substring(1)] = val;
                });
            }
            var body_1 = ctx.request['body'] || {};
            var providers = params.map(function (param, idx) {
                try {
                    var ptype = param.type;
                    if (core_1.isClass(ptype) && parser.isModel(ptype)) {
                        var val = parser.parseModel(ptype, body_1);
                        return core_1.Provider.createParam(param.name || ptype, val, idx);
                    }
                    else if (parser.isBaseType(ptype)) {
                        var paramVal = restParams_1[param.name];
                        if (core_1.isUndefined(paramVal)) {
                            paramVal = ctx.request.query[param.name];
                        }
                        if (!core_1.isUndefined(paramVal)) {
                            var val = parser.parseBaseType(ptype, paramVal);
                            return core_1.Provider.createParam(param.name, val, idx);
                        }
                        else {
                            return null;
                        }
                    }
                    else {
                        return null;
                    }
                }
                catch (err) {
                    throw new index_3.BadRequestError(err.toString());
                }
            });
            return providers.filter(function (p) { return p !== null; });
        }
        return [];
    };
    ControllerRoute.prototype.isRestUri = function (uri) {
        return /\/:/.test(uri || '');
    };
    ControllerRoute.prototype.getRouteMetaData = function (ctx, container, requestMethod) {
        var _this = this;
        var decoratorName = index_1.Route.toString();
        var baseURL = this.cutEmptyPath(this.url, true);
        var routPath = this.cutEmptyPath(ctx.url.replace(baseURL, ''));
        var methodMaps = core_1.getMethodMetadata(decoratorName, this.controller);
        var meta;
        var allMethods = [];
        for (var name_1 in methodMaps) {
            allMethods = allMethods.concat(methodMaps[name_1]);
        }
        allMethods = allMethods.filter(function (m) { return m && m.method === requestMethod; });
        allMethods = allMethods.sort(function (ra, rb) { return (rb.route || '').length - (ra.route || '').length; });
        meta = allMethods.find(function (route) { return (route.route || '') === routPath; });
        if (!meta) {
            meta = allMethods.find(function (route) {
                var uri = route.route || '';
                // if (uri === routPath) {
                //     return true;
                // }
                if (_this.isRestUri(uri)) {
                    var idex = uri.indexOf('/:');
                    var url = uri.substring(0, idex);
                    if (url !== routPath && routPath.indexOf(url) === 0) {
                        return true;
                    }
                }
                return false;
            });
        }
        return meta;
    };
    ControllerRoute.classAnnations = { "name": "ControllerRoute", "params": { "constructor": ["route", "controller"], "options": ["container", "ctx", "next"], "navigating": ["container", "ctx", "next"], "invokeOption": ["ctx", "container", "next"], "getCorsMeta": ["ctx", "container", "reqMethod"], "invoke": ["ctx", "container"], "createProvider": ["container", "ctx", "ctrl", "meta", "params"], "isRestUri": ["uri"], "getRouteMetaData": ["ctx", "container", "requestMethod"] } };
    return ControllerRoute;
}(BaseRoute_1.BaseRoute));
exports.ControllerRoute = ControllerRoute;

//# sourceMappingURL=../../sourcemaps/core/router/ControllerRoute.js.map
