"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var core_1 = require("@ts-ioc/core");
var index_1 = require("../decorators/index");
var IConfiguration_1 = require("../../IConfiguration");
var aop_1 = require("@ts-ioc/aop");
var ModelParser = /** @class */ (function () {
    function ModelParser(container, config) {
        this.container = container;
        this.config = config;
    }
    ModelParser.prototype.isModel = function (type) {
        if (this.config.modelOptions) {
            return Reflect.hasOwnMetadata(this.config.modelOptions.classMetaname, type);
        }
        else {
            return Reflect.hasOwnMetadata(index_1.Model.toString(), type);
        }
    };
    ModelParser.prototype.getPropertyMeta = function (type) {
        var meta;
        if (this.config.modelOptions) {
            meta = core_1.getPropertyMetadata(this.config.modelOptions.fieldMetaname, type);
        }
        else {
            meta = core_1.getPropertyMetadata(index_1.Field, type);
        }
        return meta;
    };
    ModelParser.prototype.parseModel = function (type, objMap) {
        var meta = this.getPropertyMeta(type);
        var result = this.container.get(type);
        for (var n in meta) {
            var propmetas = meta[n];
            if (propmetas.length) {
                if (!core_1.isUndefined(objMap[n])) {
                    var propmeta = propmetas.find(function (p) { return !!p.type; });
                    var reqval = objMap[n];
                    var parmVal = void 0;
                    if (this.isBaseType(propmeta.type)) {
                        parmVal = this.parseBaseType(propmeta.type, reqval);
                    }
                    else if (core_1.isClass(propmeta.type) && this.isModel(propmeta.type)) {
                        parmVal = this.parseModel(propmeta.type, reqval);
                    }
                    result[n] = parmVal;
                }
            }
        }
        return result;
    };
    ModelParser.prototype.parseBaseType = function (type, paramVal) {
        var val;
        if (type === String) {
            val = String(paramVal);
        }
        else if (type === Boolean) {
            val = new Boolean(paramVal);
        }
        else if (type === Number) {
            val = parseFloat(paramVal);
        }
        else if (type === Date) {
            val = new Date(paramVal);
        }
        else {
            val = paramVal;
        }
        return val;
    };
    ModelParser.prototype.isBaseType = function (p) {
        if (!core_1.isToken(p)) {
            return true;
        }
        if (p === Boolean || p === String || p === Number || p === Date) {
            return true;
        }
        return false;
    };
    ModelParser.classAnnations = { "name": "ModelParser", "params": { "constructor": ["container", "config"], "isModel": ["type"], "getPropertyMeta": ["type"], "parseModel": ["type", "objMap"], "parseBaseType": ["type", "paramVal"], "isBaseType": ["p"] } };
    ModelParser = tslib_1.__decorate([
        aop_1.NonePointcut,
        core_1.Singleton,
        tslib_1.__param(0, core_1.Inject(core_1.ContainerToken)), tslib_1.__param(1, core_1.Inject(IConfiguration_1.ConfigurationToken)),
        tslib_1.__metadata("design:paramtypes", [Object, Object])
    ], ModelParser);
    return ModelParser;
}());
exports.ModelParser = ModelParser;

//# sourceMappingURL=../../sourcemaps/core/router/ModelParser.js.map
